#!/usr/bin/python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Copyright (C) 2005 Michele Baldessari <michele@pupazzo.org>

# This simple bot connects to the SILC server on localhost:706
# joins the "bottest" channel and prints on stdout all the info
# (WHOIS and GETKEY) of people joining the channel


import time, sys, os.path, os
import silc
import silc.client

notifies = {
    silc.NOTIFY_TYPE_NONE:           "NONE",
    silc.NOTIFY_TYPE_INVITE:         "INVITE",
    silc.NOTIFY_TYPE_JOIN:           "JOIN",
    silc.NOTIFY_TYPE_LEAVE:          "LEAVE",
    silc.NOTIFY_TYPE_SIGNOFF:        "SIGNOFF",
    silc.NOTIFY_TYPE_TOPIC_SET:      "TOPIC_SET",
    silc.NOTIFY_TYPE_NICK_CHANGE:    "NICK_CHANGE",
    silc.NOTIFY_TYPE_CMODE_CHANGE:   "CMODE_CHANGE",
    silc.NOTIFY_TYPE_CUMODE_CHANGE:  "CUMODE_CHANGE",
    silc.NOTIFY_TYPE_MOTD:           "MOTD",
    silc.NOTIFY_TYPE_CHANNEL_CHANGE: "CHANNEL_CHANGE",
    silc.NOTIFY_TYPE_SERVER_SIGNOFF: "SERVER_SIGNOFF",
    silc.NOTIFY_TYPE_KICKED:         "KICKED",
    silc.NOTIFY_TYPE_KILLED:         "KILLED",
    silc.NOTIFY_TYPE_UMODE_CHANGE:   "UMODE_CHANGE",
    silc.NOTIFY_TYPE_BAN:            "BAN",
    silc.NOTIFY_TYPE_ERROR:          "ERROR",
    silc.NOTIFY_TYPE_WATCH:          "WATCH"
}

cmd_replies = {
    silc.COMMAND_NONE:     "NONE",
    silc.COMMAND_WHOIS:    "WHOIS",
    silc.COMMAND_WHOWAS:   "WHOWAS",
    silc.COMMAND_IDENTIFY: "IDENTIFY",
    silc.COMMAND_NICK:     "NICK",
    silc.COMMAND_LIST:     "LIST",
    silc.COMMAND_TOPIC:    "TOPIC",
    silc.COMMAND_INVITE:   "INVITE",
    silc.COMMAND_QUIT:     "QUIT",
    silc.COMMAND_KILL:     "KILL",
    silc.COMMAND_INFO:     "INFO",
    silc.COMMAND_STATS:    "STATS",
    silc.COMMAND_PING:     "PING",
    silc.COMMAND_OPER:     "OPER",
    silc.COMMAND_JOIN:     "JOIN",
    silc.COMMAND_MOTD:     "MOTD",
    silc.COMMAND_UMODE:    "UMODE",
    silc.COMMAND_CMODE:    "CMODE",
    silc.COMMAND_CUMODE:   "CUMODE",
    silc.COMMAND_KICK:     "KICK",
    silc.COMMAND_BAN:      "BAN",
    silc.COMMAND_DETACH:   "DETACH",
    silc.COMMAND_WATCH:    "WATCH",
    silc.COMMAND_SILCOPER: "SILCOPER",
    silc.COMMAND_LEAVE:    "LEAVE",
    silc.COMMAND_USERS:    "USERS",
    silc.COMMAND_GETKEY:   "GETKEY",
    silc.COMMAND_SERVICE:  "SERVICE"
}

def cb_say(msg):
    print "SAY : ", msg

def cb_connected(msg):
    print "CONNECTED : ", msg
    s.command("JOIN bottest")
    s.command("STATS")

def cb_disconnected(a, b):
    print "DISCONNECTED"
    sys.exit(0)

def cb_command_reply(code, *msg):
    print "COMMAND REPLY %d <%s>: %s" % (code, cmd_replies[code], msg)
    if code == silc.COMMAND_JOIN:
        s.send_channel("bottest", "CUCU")

def cb_channel_message(sender, msg):
    print "MSG : <%s> %s" % (sender, msg)
    if msg.startswith(s.username):
        s.send_channel("bottest", "hello", signed=False)

def cb_private_message(sender, msg):
    print "PRIVATE : <%s> %s" % (sender, msg)

def cb_failure(status, msg):
    print "FAILURE %d: %s" % (status, msg)

def cb_notify(code, *args):
    print "NOTIFY %d <%s> : %s" % (code, notifies[code], args)
    if code == silc.NOTIFY_TYPE_JOIN:
        s.command("WHOIS %s" % args[1])
        s.command("GETKEY %s" % args[1])
    
callbacks = {
    "connected":       cb_connected,
    "disconnected":    cb_disconnected,
    "command_reply":   cb_command_reply,
    "channel_message": cb_channel_message,
    "private_message": cb_private_message,
    "failure":         cb_failure,
    "say":             cb_say,
    "notify":          cb_notify
}

try:
    os.unlink("test.pub")
    os.unlink("test.prv")
except:
    pass

s = silc.client.Client()
s.username = "pybot"
s.realname = "i am the pybot"
s.hostname = "localhost"
s.createkeys("test.pub", "test.prv", "foobarness")
#s.createkeys("test.pub", "test.prv", "foobarness", identifier="", pkcs="rsa", pkcs_len=2048)
s.setcallbacks(callbacks)
s.connect("localhost", port=706)

while 1:
    try:
        s.runone()
    except silclient.error:
        traceback.print_exc()
        break
    time.sleep(0.5)

try:
    os.unlink("test.pub")
    os.unlink("test.prv")
except:
    pass
