/*-*- Mode: C -*-
 vi:si:et:sw=4:sts=4:ts=4 
 */

#include <Python.h>
#include "silcincludes.h"
#include "silcclient.h"

#define TRUE  1
#define FALSE 0


PyObject *SilcError = NULL;

static PyMethodDef silc_methods[] = {
        {NULL, NULL, 0, NULL}        /* Sentinel */
};

static struct {
        char *name;
            int constant;
} int_consts[] = {
    /* Notify Types */
    { "NOTIFY_TYPE_NONE",           SILC_NOTIFY_TYPE_NONE },
    { "NOTIFY_TYPE_INVITE",         SILC_NOTIFY_TYPE_INVITE },
    { "NOTIFY_TYPE_JOIN",           SILC_NOTIFY_TYPE_JOIN },
    { "NOTIFY_TYPE_LEAVE",          SILC_NOTIFY_TYPE_LEAVE },
    { "NOTIFY_TYPE_SIGNOFF",        SILC_NOTIFY_TYPE_SIGNOFF },
    { "NOTIFY_TYPE_TOPIC_SET",      SILC_NOTIFY_TYPE_TOPIC_SET },
    { "NOTIFY_TYPE_NICK_CHANGE",    SILC_NOTIFY_TYPE_NICK_CHANGE },
    { "NOTIFY_TYPE_CMODE_CHANGE",   SILC_NOTIFY_TYPE_CMODE_CHANGE },
    { "NOTIFY_TYPE_CUMODE_CHANGE",  SILC_NOTIFY_TYPE_CUMODE_CHANGE },
    { "NOTIFY_TYPE_MOTD",           SILC_NOTIFY_TYPE_MOTD },
    { "NOTIFY_TYPE_CHANNEL_CHANGE", SILC_NOTIFY_TYPE_CHANNEL_CHANGE },
    { "NOTIFY_TYPE_SERVER_SIGNOFF", SILC_NOTIFY_TYPE_SERVER_SIGNOFF },
    { "NOTIFY_TYPE_KICKED",         SILC_NOTIFY_TYPE_KICKED },
    { "NOTIFY_TYPE_KILLED",         SILC_NOTIFY_TYPE_KILLED },
    { "NOTIFY_TYPE_UMODE_CHANGE",   SILC_NOTIFY_TYPE_UMODE_CHANGE },
    { "NOTIFY_TYPE_BAN",            SILC_NOTIFY_TYPE_BAN },
    { "NOTIFY_TYPE_ERROR",          SILC_NOTIFY_TYPE_ERROR },
    { "NOTIFY_TYPE_WATCH",          SILC_NOTIFY_TYPE_WATCH },

    /* Commands */
    { "COMMAND_NONE",     SILC_COMMAND_NONE },
    { "COMMAND_WHOIS",    SILC_COMMAND_WHOIS },
    { "COMMAND_WHOWAS",   SILC_COMMAND_WHOWAS },
    { "COMMAND_IDENTIFY", SILC_COMMAND_IDENTIFY },
    { "COMMAND_NICK",     SILC_COMMAND_NICK },
    { "COMMAND_LIST",     SILC_COMMAND_LIST },
    { "COMMAND_TOPIC",    SILC_COMMAND_TOPIC },
    { "COMMAND_INVITE",   SILC_COMMAND_INVITE },
    { "COMMAND_QUIT",     SILC_COMMAND_QUIT },
    { "COMMAND_KILL",     SILC_COMMAND_KILL },
    { "COMMAND_INFO",     SILC_COMMAND_INFO },
    { "COMMAND_STATS",    SILC_COMMAND_STATS },
    { "COMMAND_PING",     SILC_COMMAND_PING },
    { "COMMAND_OPER",     SILC_COMMAND_OPER },
    { "COMMAND_JOIN",     SILC_COMMAND_JOIN },
    { "COMMAND_MOTD",     SILC_COMMAND_MOTD },
    { "COMMAND_UMODE",    SILC_COMMAND_UMODE },
    { "COMMAND_CMODE",    SILC_COMMAND_CMODE },
    { "COMMAND_CUMODE",   SILC_COMMAND_CUMODE },
    { "COMMAND_KICK",     SILC_COMMAND_KICK },
    { "COMMAND_BAN",      SILC_COMMAND_BAN },
    { "COMMAND_DETACH",   SILC_COMMAND_DETACH },
    { "COMMAND_WATCH",    SILC_COMMAND_WATCH },
    { "COMMAND_SILCOPER", SILC_COMMAND_SILCOPER },
    { "COMMAND_LEAVE",    SILC_COMMAND_LEAVE },
    { "COMMAND_USERS",    SILC_COMMAND_USERS },
    { "COMMAND_GETKEY",   SILC_COMMAND_GETKEY },
    { "COMMAND_SERVICE",  SILC_COMMAND_SERVICE },

    /* Client Status */
    { "CLIENT_CONN_SUCCESS",        SILC_CLIENT_CONN_SUCCESS },
    { "CLIENT_CONN_SUCCESS_RESUME", SILC_CLIENT_CONN_SUCCESS_RESUME },
    { "CLIENT_CONN_ERROR",          SILC_CLIENT_CONN_ERROR },
    { "CLIENT_CONN_ERROR_KE",       SILC_CLIENT_CONN_ERROR_KE },
    { "CLIENT_CONN_ERROR_AUTH",     SILC_CLIENT_CONN_ERROR_AUTH },
    { "CLIENT_CONN_ERROR_RESUME",   SILC_CLIENT_CONN_ERROR_RESUME },
    { "CLIENT_CONN_ERROR_TIMEOUT",  SILC_CLIENT_CONN_ERROR_TIMEOUT }
};

void init_silc() {
    PyObject *m;
    int i;

    m = Py_InitModule("_silc", silc_methods);

    for (i = 0; i < sizeof(int_consts) / sizeof(int_consts[0]); i++) {
        PyModule_AddIntConstant(m, int_consts[i].name, int_consts[i].constant);
    }

    SilcError = PyErr_NewException("silc.Error", NULL, NULL);
    Py_INCREF(SilcError);
    PyModule_AddObject(m, "SilcError", SilcError);
                
}
